@@ -59,7 +59,7 @@ editor_shell_new_from_theme (MtmEnv *env, MtmTheme *theme, GtkWidget *parent_win
 	g_return_val_if_fail (MTM_IS_THEME (theme), NULL);
 	
 	shell = editor_shell_new (env);
-	gtk_object_set_data (GTK_OBJECT (shell), "theme", theme);
+	g_object_set_data (G_OBJECT (shell), "theme", theme);
 
 	l = g_list_copy (mtm_theme_get_exts (theme));
 	l = g_list_sort (l, key_compare_func);
