@@ -53,15 +53,15 @@ mtm_env_destroy (GtkObject *object)
 
 /* Class initialization function for MtmEnv */
 static void
-mtm_env_class_init (GtkObjectClass *object_class)
+mtm_env_class_init (GObjectClass *object_class)
 {
 	mtm_env_parent_class = gtk_type_class (gtk_object_get_type ());
-	object_class->destroy = mtm_env_destroy;
+	GTK_OBJECT_CLASS (object_class)->destroy = mtm_env_destroy;
 }
 
 /* Object initialization function for MtmEnv */
 static void
-mtm_env_init (GtkObject *object)
+mtm_env_init (GObject *object)
 {
 	MtmEnv *env = MTM_ENV (object);
 	gchar *home;
@@ -90,26 +90,30 @@ mtm_env_init (GtkObject *object)
  * 
  * Return value: The type ID of the #MtmEnv class.
  */
-GtkType
+GType
 mtm_env_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 
 	if (!type)
 	{
-		GtkTypeInfo info =
+		GTypeInfo info =
 		{
-			"MtmEnv",
-			sizeof (MtmEnv),
 			sizeof (MtmEnvClass),
-			(GtkClassInitFunc) mtm_env_class_init,
-			(GtkObjectInitFunc) mtm_env_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL
+			NULL,
+			NULL,
+			(GClassInitFunc) mtm_env_class_init,
+			NULL,
+			NULL,
+			sizeof (MtmEnv),
+			0,	
+			(GInstanceInitFunc) mtm_env_init,
+			NULL
 		};
 
-		type = gtk_type_unique (gtk_object_get_type (), &info);
+		type = g_type_register_static (GTK_TYPE_OBJECT,
+					       "MtmEnv",
+					       &info, 0);
 	}
 
 	return type;
@@ -126,7 +130,7 @@ mtm_env_get_type (void)
 MtmEnv *
 mtm_env_new (void)
 {
-	return gtk_type_new (mtm_env_get_type ());
+	return g_object_new (mtm_env_get_type (), NULL);
 }
 
 const gchar *
