@@ -38,7 +38,7 @@
 
 #include <gdk/gdk.h>
 
-static MtmStatefulClass *mtm_theme_parent_class;
+static GObjectClass *mtm_theme_parent_class;
 
 struct _MtmThemePrivate {
 	GList *exts;
@@ -55,69 +55,107 @@ enum
 	ARG_AUTHOR_EMAIL
 };
 
-static void mtm_theme_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void mtm_theme_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
+static void mtm_theme_set_arg (GObject *object, guint arg_id, const GValue *value, GParamSpec *spec);
+static void mtm_theme_get_arg (GObject *object, guint arg_id, GValue *value, GParamSpec *spec);
 
 static void parse_args (MtmExt *ext, xmlNodePtr node);
 
+#define FREE_AND_NULL(obj) if (obj) { g_free (obj); obj = NULL; }
+
 static void
-mtm_theme_destroy (GtkObject *object)
+mtm_theme_dispose (GObject *object)
 {
 	MtmTheme *theme = MTM_THEME (object);
 	GList *list = NULL;
 
-	if (theme->desc)
-		g_free (theme->desc);
-	if (theme->name)
-		g_free (theme->name);
-	if (theme->filename)
-		g_free (theme->filename);
-	if (theme->preview)
-		g_free (theme->preview);
-	if (theme->author)
-		g_free (theme->author);
-	if (theme->author_email)
-		g_free (theme->author_email);
+	FREE_AND_NULL (theme->desc);
+	FREE_AND_NULL (theme->name);
+	FREE_AND_NULL (theme->filename);
+	FREE_AND_NULL (theme->preview);
+	FREE_AND_NULL (theme->author);
+	FREE_AND_NULL (theme->author_email);
 	
-	list = theme->priv->exts;
-	while (list)
+	if (theme->priv->exts)
 	{
-		gtk_object_unref (GTK_OBJECT (list->data));
-		list = list->next;
+		list = theme->priv->exts;
+		while (list)
+		{
+			g_object_unref (G_OBJECT (list->data));
+			list = list->next;
+		}
+		g_list_free (theme->priv->exts);
+		theme->priv->exts = NULL;
 	}
-	g_list_free (theme->priv->exts);
-	g_free (theme->priv);
 
-	if (GTK_OBJECT_CLASS(mtm_theme_parent_class)->destroy)
-		(*GTK_OBJECT_CLASS(mtm_theme_parent_class)->destroy)(object);
+	mtm_theme_parent_class->dispose (object);
 }
 
 static void
-mtm_theme_class_init (GtkObjectClass *object_class)
+mtm_theme_finalize (GObject *object)
+{
+	MtmTheme *theme = MTM_THEME (object);
+	
+	g_free (theme->priv);
+	
+	mtm_theme_parent_class->finalize (object);
+}
+
+static void
+mtm_theme_class_init (GObjectClass *object_class)
 {
 	mtm_theme_parent_class = gtk_type_class (mtm_stateful_get_type ());
 
-	object_class->set_arg = mtm_theme_set_arg;
-	object_class->get_arg = mtm_theme_get_arg;
+	object_class->set_property = mtm_theme_set_arg;
+	object_class->get_property = mtm_theme_get_arg;
+	object_class->dispose = mtm_theme_dispose;
+	object_class->finalize = mtm_theme_finalize;
 
-	gtk_object_add_arg_type ("MtmTheme::desc", GTK_TYPE_STRING,
-						GTK_ARG_READWRITE, ARG_DESC);
-	gtk_object_add_arg_type ("MtmTheme::name", GTK_TYPE_STRING,
-						GTK_ARG_READWRITE, ARG_NAME);
-	gtk_object_add_arg_type ("MtmTheme::filename", GTK_TYPE_STRING,
-						GTK_ARG_READWRITE, ARG_FILENAME);
-	gtk_object_add_arg_type ("MtmTheme::preview", GTK_TYPE_STRING,
-						GTK_ARG_READWRITE, ARG_PREVIEW);
-	gtk_object_add_arg_type ("MtmTheme::author", GTK_TYPE_STRING,
-						GTK_ARG_READWRITE, ARG_AUTHOR);
-	gtk_object_add_arg_type ("MtmTheme::author_email", GTK_TYPE_STRING,
-						GTK_ARG_READWRITE, ARG_AUTHOR_EMAIL);
-
-	object_class->destroy = mtm_theme_destroy;
+	g_object_class_install_property
+		(object_class, ARG_DESC,
+		 g_param_spec_string ("desc",
+			 	      "desc",
+				      "desc",
+				      NULL,
+				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_NAME,
+		 g_param_spec_string ("name",
+			 	      "name",
+				      "name",
+				      NULL,
+				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_FILENAME,
+		 g_param_spec_string ("filename",
+			 	      "filename",
+				      "filename",
+				      NULL,
+				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_PREVIEW,
+		 g_param_spec_string ("preview",
+			 	      "preview",
+				      "preview",
+				      NULL,
+				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_AUTHOR,
+		 g_param_spec_string ("author",
+			 	      "author",
+				      "author",
+				      NULL,
+				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_AUTHOR_EMAIL,
+		 g_param_spec_string ("author_email",
+			 	      "author_email",
+				      "author_email",
+				      NULL,
+				      G_PARAM_READWRITE));
 }
 
 static void
-mtm_theme_init (GtkObject *object)
+mtm_theme_init (GObject *object)
 {
 	MtmTheme *theme = MTM_THEME (object);
 
@@ -133,26 +171,30 @@ mtm_theme_init (GtkObject *object)
  * 
  * Return value: The type ID of the #MtmTheme class.
  **/
-GtkType
+GType
 mtm_theme_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 
 	if (!type)
 	{
-		GtkTypeInfo info =
+		GTypeInfo info =
 		{
-			"MtmTheme",
-			sizeof (MtmTheme),
 			sizeof (MtmThemeClass),
-			(GtkClassInitFunc) mtm_theme_class_init,
-			(GtkObjectInitFunc) mtm_theme_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL
+			NULL,
+			NULL,
+			(GClassInitFunc) mtm_theme_class_init,
+			NULL,
+			NULL,
+			sizeof (MtmTheme),
+			0,	
+			(GInstanceInitFunc) mtm_theme_init,
+			NULL
 		};
 
-		type = gtk_type_unique (mtm_stateful_get_type (), &info);
+		type = g_type_register_static (MTM_STATEFUL_TYPE,
+					       "MtmTheme",
+					       &info, 0);
 	}
 
 	return type;
@@ -187,7 +229,7 @@ mtm_theme_activate (MtmTheme *theme)
 			continue;
 
 		/*FIXME: Evil hack */
-		if (gtk_object_get_data (GTK_OBJECT (ext), "default"))
+		if (g_object_get_data (G_OBJECT (ext), "default"))
 			pas = NULL;
 		else
 			pas = ext;
@@ -196,8 +238,8 @@ mtm_theme_activate (MtmTheme *theme)
 
 		if (ext->handler->n_subclass_args)
 		{
-			gtk_object_set (GTK_OBJECT (ext->handler),
-						 "ext_context", ext, NULL);
+			g_object_set (G_OBJECT (ext->handler),
+				      "ext_context", ext, NULL);
 		}
 		ret = ext->handler->activate (ext->handler, pas);
 	}
@@ -233,8 +275,8 @@ mtm_theme_update_exts (MtmTheme *theme)
 		
 		if (ext->handler->n_subclass_args)
 		{
-			gtk_object_set (GTK_OBJECT (ext->handler),
-					"ext_context", ext, NULL);
+			g_object_set (G_OBJECT (ext->handler),
+				      "ext_context", ext, NULL);
 		}
 		ret = ext->handler->update_ext (ext->handler);
 	}
@@ -265,7 +307,7 @@ mtm_theme_add_ext (MtmTheme *theme, MtmExt *ext)
 	g_return_if_fail (ext != NULL);
 	
 	theme->priv->exts = g_list_append (theme->priv->exts, ext);
-	gtk_object_ref (GTK_OBJECT (ext));
+	g_object_ref (G_OBJECT (ext));
 	gtk_object_sink (GTK_OBJECT (ext));
 }
 
@@ -286,7 +328,7 @@ mtm_theme_remove_ext (MtmTheme *theme, MtmExt *ext)
 			
 			theme->priv->exts = g_list_remove_link (
 				theme->priv->exts, l);
-			gtk_object_unref (GTK_OBJECT (ext));
+			g_object_unref (G_OBJECT (ext));
 		}
 		l = l2;
 	}
@@ -299,7 +341,7 @@ mtm_theme_new (MtmEnv *env)
 
 	g_return_val_if_fail (env != NULL, NULL);
 	
-	theme = gtk_type_new (mtm_theme_get_type ());
+	theme = g_object_new (mtm_theme_get_type (), NULL);
 	mtm_theme_construct (theme, env);
 
 	return theme;
@@ -314,7 +356,7 @@ mtm_theme_new_from_file (MtmEnv *env, const gchar *filename)
 	g_return_val_if_fail (env != NULL, NULL);
 	g_return_val_if_fail (filename != NULL, NULL); 
 
-	theme = gtk_type_new (mtm_theme_get_type ());
+	theme = g_object_new (mtm_theme_get_type (), NULL);
 	mtm_theme_construct_from_file (theme, env, filename);
 	if (!theme->filename)
 	{
@@ -332,7 +374,7 @@ mtm_theme_new_from_defaults (MtmEnv *env)
 
 	g_return_val_if_fail (MTM_IS_ENV (env), NULL);
 
-	theme = gtk_type_new (mtm_theme_get_type ());
+	theme = g_object_new (mtm_theme_get_type (), NULL);
 	mtm_theme_construct_from_defaults (theme, env);
 
 	return theme;
@@ -390,8 +432,8 @@ mtm_theme_construct_extension_from_node (xmlNodePtr node, MtmEnv *env, const gch
 	
 	if (node->children && ext->handler)
 	{
-		gtk_object_set (GTK_OBJECT (ext->handler),
-					 "ext_context", ext, NULL);
+		g_object_set (G_OBJECT (ext->handler),
+			      "ext_context", ext, NULL);
 		parse_args (ext, node->children);
 	}
 	
@@ -495,7 +537,7 @@ mtm_theme_construct_from_defaults (MtmTheme *theme, MtmEnv *env)
 }
 
 MtmResult
-mtm_theme_save_as (MtmTheme *theme, gchar *filename)
+mtm_theme_save_as (MtmTheme *theme, const gchar *filename)
 {
 	char *indent = "\t";
 	FILE *file;
@@ -521,11 +563,13 @@ mtm_theme_save_as (MtmTheme *theme, gchar *filename)
 		fprintf (file, "%s<desc>%s</desc>\n", indent, theme->desc);
 	if (theme->preview)
 	{
-		gchar *tofile = g_strconcat (filename, "/", g_basename (theme->preview), NULL);
+		gchar *base = g_path_get_basename (theme->preview);
+		gchar *tofile = g_strconcat (filename, "/", base, NULL);
 		
-		fprintf (file, "%s<preview>%s</preview>\n", indent, g_basename (theme->preview));
+		fprintf (file, "%s<preview>%s</preview>\n", indent, base);
 		mtm_copy_file (theme->preview, tofile);
 		g_free (tofile);
+		g_free (base);
 	}
 	if (theme->author)
 	{
@@ -544,14 +588,16 @@ mtm_theme_save_as (MtmTheme *theme, gchar *filename)
 		if (ext->file)
 		{
 			gchar *tmp;
+			gchar *base = g_path_get_basename (ext->file); 
 			
 			if (mtm_file_is_targz (ext->file))
-				tmp = mtm_strip_ext (g_basename (ext->file), ".tar.gz");
+				tmp = mtm_strip_ext (base, ".tar.gz");
 			else
-				tmp = g_strdup (g_basename (ext->file));
+				tmp = g_strdup (base);
 			fprintf (file, " file=\"%s\"", tmp);
 			
 			g_free (tmp);
+			g_free (base);
 		}
 		
 		if (ext->handler && ext->handler->subclass_args)
@@ -564,8 +610,8 @@ mtm_theme_save_as (MtmTheme *theme, gchar *filename)
 			
 			fprintf (file, ">\n");
 
-			gtk_object_set (GTK_OBJECT (ext->handler),
-				"ext_context", ext, NULL);
+			g_object_set (G_OBJECT (ext->handler),
+				      "ext_context", ext, NULL);
 
 			for (i = 0; i < n; i++)
 			{
@@ -589,12 +635,14 @@ mtm_theme_save_as (MtmTheme *theme, gchar *filename)
 		/* Save the actual extension */
 		if (ext->handler && ext->handler->save && ext->file)
 		{
+			gchar *base = g_path_get_basename (ext->file);
 			gchar *todir = g_strconcat (filename, "/", ext->type, NULL);
-			gchar *savefile = g_strconcat (todir, "/", g_basename (ext->file), NULL);
+			gchar *savefile = g_strconcat (todir, "/", base, NULL);
 			
 			ext->handler->save (ext->handler, ext, todir);
 			g_free (todir);
 			g_free (savefile);
+			g_free (base);
 		}
 		
 		exts = exts->next;
@@ -616,57 +664,57 @@ mtm_theme_save (MtmTheme *theme)
 }
 
 static void
-mtm_theme_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+mtm_theme_set_arg (GObject *object, guint arg_id, const GValue *value, GParamSpec *spec)
 {
 	MtmTheme *theme = MTM_THEME (object);
 
 	switch (arg_id)
 	{
 		case ARG_DESC:
-			mtm_theme_set_desc (theme, GTK_VALUE_STRING (*arg));
+			mtm_theme_set_desc (theme, g_value_get_string (value));
 			break;
 		case ARG_NAME:
-			mtm_theme_set_name (theme, GTK_VALUE_STRING (*arg));
+			mtm_theme_set_name (theme, g_value_get_string (value));
 			break;
 		case ARG_FILENAME:
-			mtm_theme_set_filename (theme, GTK_VALUE_STRING (*arg));
+			mtm_theme_set_filename (theme, g_value_get_string (value));
 			break;
 		case ARG_PREVIEW:
-			mtm_theme_set_preview (theme, GTK_VALUE_STRING (*arg));
+			mtm_theme_set_preview (theme, g_value_get_string (value));
 			break;
 		case ARG_AUTHOR:
-			mtm_theme_set_author (theme, GTK_VALUE_STRING (*arg));
+			mtm_theme_set_author (theme, g_value_get_string (value));
 			break;
 		case ARG_AUTHOR_EMAIL:
-			mtm_theme_set_author_email (theme, GTK_VALUE_STRING (*arg));
+			mtm_theme_set_author_email (theme, g_value_get_string (value));
 			break;
 	}
 }
 
 static void
-mtm_theme_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+mtm_theme_get_arg (GObject *object, guint arg_id, GValue *value, GParamSpec *spec)
 {
 	MtmTheme *theme = MTM_THEME (object);
 
 	switch (arg_id)
 	{
 		case ARG_DESC:
-			GTK_VALUE_STRING (*arg) = (gchar *) mtm_theme_get_desc (theme);
+			g_value_set_string (value, mtm_theme_get_desc (theme));
 			break;
 		case ARG_NAME:
-			GTK_VALUE_STRING (*arg) = (gchar *) mtm_theme_get_name (theme);
+			g_value_set_string (value, mtm_theme_get_name (theme));
 			break;
 		case ARG_FILENAME:
-			GTK_VALUE_STRING (*arg) = (gchar *) mtm_theme_get_filename (theme);;
+			g_value_set_string (value, mtm_theme_get_filename (theme));
 			break;
 		case ARG_PREVIEW:
-			GTK_VALUE_STRING (*arg) = (gchar *) mtm_theme_get_preview (theme);;
+			g_value_set_string (value, mtm_theme_get_preview (theme));
 			break;
 		case ARG_AUTHOR:
-			GTK_VALUE_STRING (*arg) = (gchar *) mtm_theme_get_author (theme);;
+			g_value_set_string (value, mtm_theme_get_author (theme));
 			break;
 		case ARG_AUTHOR_EMAIL:
-			GTK_VALUE_STRING (*arg) = (gchar *) mtm_theme_get_author_email (theme);;
+			g_value_set_string (value, mtm_theme_get_author_email (theme));
 			break;
 	}
 }
@@ -683,7 +731,7 @@ mtm_theme_get_desc (MtmTheme *theme)
 }
 
 void
-mtm_theme_set_desc (MtmTheme *theme, gchar *desc)
+mtm_theme_set_desc (MtmTheme *theme, const gchar *desc)
 {
 	g_return_if_fail (MTM_IS_THEME (theme));
 
@@ -702,7 +750,7 @@ mtm_theme_get_name (MtmTheme *theme)
 }
 
 void
-mtm_theme_set_name (MtmTheme *theme, gchar *name)
+mtm_theme_set_name (MtmTheme *theme, const gchar *name)
 {
 	g_return_if_fail (MTM_IS_THEME (theme));
 	
@@ -720,7 +768,7 @@ mtm_theme_get_filename (MtmTheme *theme)
 }
 
 void
-mtm_theme_set_filename (MtmTheme *theme, gchar *filename)
+mtm_theme_set_filename (MtmTheme *theme, const gchar *filename)
 {
 	int len;
 	
@@ -751,7 +799,7 @@ mtm_theme_get_preview (MtmTheme *theme)
 }
 
 void
-mtm_theme_set_preview (MtmTheme *theme, gchar *preview)
+mtm_theme_set_preview (MtmTheme *theme, const gchar *preview)
 {
 	g_return_if_fail (MTM_IS_THEME (theme));
 
@@ -769,7 +817,7 @@ mtm_theme_get_author (MtmTheme *theme)
 }
 
 void
-mtm_theme_set_author (MtmTheme *theme, gchar *author)
+mtm_theme_set_author (MtmTheme *theme, const gchar *author)
 {
 	g_return_if_fail (MTM_IS_THEME (theme));
 	
@@ -786,7 +834,7 @@ mtm_theme_get_author_email (MtmTheme *theme)
 	return theme->author_email;
 }
 
-void mtm_theme_set_author_email (MtmTheme *theme, gchar *author_email)
+void mtm_theme_set_author_email (MtmTheme *theme, const gchar *author_email)
 {
 	g_return_if_fail (MTM_IS_THEME (theme));
 	
@@ -799,7 +847,7 @@ static void
 parse_args (MtmExt *ext, xmlNodePtr node)
 {
 	GdkColor color;
-	GtkType type;
+	GType type;
 	gchar *typename;
 	gchar *tmp, *name;
 
@@ -811,8 +859,8 @@ parse_args (MtmExt *ext, xmlNodePtr node)
 			GValue arg = { 0, };
 			
 			typename = xmlGetProp (node, "type");
-			type = gtk_type_from_name (typename);
-			if (type == GTK_TYPE_INVALID)
+			type = g_type_from_name (typename);
+			if (type == G_TYPE_INVALID)
 			{
 				xmlFree (typename);
 				node = node->next;
@@ -822,14 +870,14 @@ parse_args (MtmExt *ext, xmlNodePtr node)
 			name = g_strdup (xmltmp);
 			xmlFree (xmltmp);
 			tmp = xmlGetProp (node, "value");
-			if (type == GTK_TYPE_STRING)
+			if (type == G_TYPE_STRING)
 			{
-				g_value_init (&arg, GTK_TYPE_STRING);
+				g_value_init (&arg, G_TYPE_STRING);
 				g_value_set_string (&arg, tmp);
 			}
-			else if (type == GTK_TYPE_BOOL)
+			else if (type == G_TYPE_BOOLEAN)
 			{
-				g_value_init (&arg, GTK_TYPE_BOOL);
+				g_value_init (&arg, G_TYPE_BOOLEAN);
 				g_value_set_boolean (&arg, tmp[0] != '0');
 			}
 			else if (type == GDK_TYPE_COLOR)
@@ -857,22 +905,28 @@ gint
 mtm_theme_compare (gconstpointer a, gconstpointer b)
 {
 	const MtmTheme *theme1, *theme2;
-	const gchar *str1, *str2;
+	gchar *str1, *str2;
+	gint result;
 	
 	theme1 = a;
 	theme2 = b;
 
 	if (theme1->name)
-		str1 = theme1->name;
+		str1 = g_strdup (theme1->name);
 	else
-		str1 = g_basename (theme1->filename);
+		str1 = g_path_get_basename (theme1->filename);
 
 	if (theme2->name)
-		str2 = theme2->name;
+		str2 = g_strdup (theme2->name);
 	else
-		str2 = g_basename (theme2->filename);
+		str2 = g_path_get_basename (theme2->filename);
 
-	return g_strcasecmp (str1, str2);
+	result = g_ascii_strcasecmp (str1, str2);
+	
+	g_free (str1);
+	g_free (str2);
+
+	return result;
 }
 
 gchar *
@@ -881,7 +935,7 @@ mtm_theme_dup_name (MtmTheme *theme)
 	g_return_val_if_fail (MTM_IS_THEME (theme), g_strdup("Error"));
 
 	if (!theme->name)
-		return g_strdup (g_basename (theme->filename));
+		return g_path_get_basename (theme->filename);
 
 	return g_strdup (theme->name);
 }
